import Lib;
import Protocol;
using MAll;

class Hex  {//}
	
	public static var RAY = 16;	
	public static var DIR = [[1, 0], [1, 1], [0, 1], [ -1, 0], [ -1, -1], [0, -1]];	
	
	public var border:Bool;
	
	public var temp:Int;
	public var digDist:Int;
	public var escapeDist:Int;
	public var ladderDist:Int;
	public var castleDist:Int;
	public var lichDist:Int;
	
	public var nei:Array<Hex>;	
	public var dnei:Array<Hex>;
	
	public var type:HexType;
	public var tile:EL;

	
	public var level:Level;
	public var x:Int;
	public var y:Int;
	public var building:Building;
	public var ents:Array<Ent>;

	
	public var bt:BonusType;
	
	public function new(lvl,x,y) {

		this.level = lvl;
		this.x = x;
		this.y = y;
		nei = [];
		dnei = [];
		ents = [];
		digDist = ladderDist = 999;
		
		tile = new EL();
		level.dm.add(tile, DP_HEX);
		center(tile);
		
	}
	
	public function setType(t,?bt) {
		type = t;
		tile.goto(Type.enumIndex(type), "hex");

	}
	

	
	
	public function center(mc:SP ) {
		var pos = getPos();
		mc.x = pos.x;
		mc.y = pos.y;
	}
	
	public inline function getPos() {
		return getHexPos(x, y);
	}
	public static function getHexPos(x,y) {
		return {
			x : (Game.WIDTH>>1) + x * 32 - y * 16,
			y : (Game.HEIGHT>>1) + y*24,
		}
	}
	
	var debugField:TF;
	public function showText(str) {
		if ( debugField == null ) {
			debugField = Tools.getField(0xFFFFFF);
			tile.addChild(debugField);

		}
		debugField.text = str;
		debugField.autosize();
		debugField.x = -Std.int(debugField.textWidth*.5);
	}
	
	
	//
	var rollContent:SP;
	public function rollOver() {
		rollContent = new SP();
		center(rollContent);
		level.dm.add(rollContent, DP_FX);
		
		var bt = this.getBuildingType();
		
		var getIcon = function(?n:Int, ?str:String)  {
			var el = new EL();
			el.goto(n,str);
			rollContent.addChild(el);
			return el;
		}
		
		if ( bt == null ) {
			switch(type) {
				case GRASS :
					getIcon(1, "tools");
					
				default :
			}
			
		}else {
			switch(bt) {
				case TOMB, TOMB_OPEN :
					Game.me.showHint( building.bdata.desc, building.bdata.name );
				case STONES : 
					
					if ( Game.me.dynamite ) {
						var el = getIcon(3, "tools");
						el.y -= 8;
					}else{
						var dt = Game.me.digTarget;
						var el = getIcon(dt != null?2:0, "tools");
						el.y -= 8;					
						el.visible = dt != this;
					}
					
				case CASTLE, LADDER :
					var el = getIcon(bt == CASTLE?0:1, "show_arrow");
					el.y = -36;
				default :
				
			}
		}
		
		
		
	}
	public function rollOut() {
		Game.me.hideHint();
		if ( rollContent == null ) return;
		rollContent.parent.removeChild(rollContent);
		rollContent = null;
	}

	// GET
	public function getZone(ray) {
		return propagate(this, function(h) return true, ray+1);
	}
	public function getEnts(ray,?team:Null<Int>) {
		var all = [];
		var zone = getZone(ray);
		for ( h in zone ) for ( e in h.ents ) if( team == null || e.team == team ) all.push(e);
		return all;
	}
	public function getBuildingType() {
		if ( building == null ) return null;
		return building.data.id;
	}
	
	// IS
	public function isFree() {
		return (type == GRASS || type == DIRT ) && ( building == null || !building.block );
	}
	public function is(bt) {
		return getBuildingType() == bt;
	}
	// FX
	public function fxStone() {
		for ( i in 0...6 ) {
			var el = new EL();
			el.goto(i % 4, "stones");
			tile.addChild(el);
			var p = new fx.Part(el);
			p.weight = 0.2 + Math.random() * 0.2;
			p.x = Std.random(RAY * 2) - RAY ;
			p.y = 8 - i * 2;
			p.setGround(p.y, 0.75, 0.7);
			p.y -= 8 + Std.random(16);
			p.onBounceGround = function() {
				p.onBounceGround = null;
				p.life = 30;
			}
		}
			
	}


	// TOOL
	public function getSwap() {
		var lvl = level.surface?Game.me.underground:Game.me.surface;
		return lvl.getHex( x, y);
	}
	public function blowUp() {
		var a = [this].concat(nei);
		for ( h in a ) {
			if ( h.building != null ) h.building.destroy();
			h.fxStone();
		}
	}
	
	// STATIC
	public static function propagate(?hex:Hex,?a:Array<Hex>,check:Hex->Bool, maximum=999) {
		if ( hex != null ) a = [hex];
		else			 hex = a[0];
		
		var all = [];
		for ( h in hex.level.grid ) h.temp = maximum;
		var work = a.copy();
		for ( h in work ) h.temp = 0;
		
		while (work.length > 0) {			
			var h = work.shift();
			all.push(h);
			for ( nh in h.nei ) {
				var inc = 1;
				if ( check(nh) && nh.temp > h.temp + inc) {
					nh.temp = h.temp + inc;
					work.push(nh);
				}
			}
		}
		return all;
		
	}
	
	
	
	//
	var digIcon:EL;
	public function setDigIcon() {
		digIcon = new EL();
		digIcon.goto("tools");
		level.dm.add(digIcon, DP_FX);
		center(digIcon);
		digIcon.y -= 8;
	}
	public function removeDigIcon() {
		digIcon.kill();
	}
	
	
//{
}






